# ---------------------------------------------
# Formant analysis report
# ---------------------------------------------
# This script make a formantic analysis of a part of the signal
# and produce a report with several type of representation of this analysis
# 
# You must put in this script into editor menu
# Just select the right part of the signal, and apply this script
# This script can place the output report in the clipboard or in a file (PDF or EPS)
# tit make also a sound object with the name of the original sound and temporal extraction informations
# ---------------------------------------------
# PRAAT script 
# Pascal Gaillard 2012
# pascal.gaillard@univ-tlse2.fr
# version 1.0 (sept 20012)
# ---------------------------------------------
# Output : PDF or EPS file
# --------------- Parameters
form Formant parameters
	sentence analysis_output_directory ~/Desktop/
	optionmenu save_table_analysis: 1
		option No
		option Yes
	optionmenu scatter_plot: 1
		option F1 F2
		option F1 F3
		option F2 F3
	optionmenu type_of_image: 2
		option Image (copy to clipboard)
		option Vectoriel (copy to clipboard)
		option PostScript (write on disk)
		option PDF (write on disk)
endform

# ----------------
tempsoundname$ = "untitled"
originalsoundname$ = selected$("Sound")

# -----------------default analysis parameters
max_number_formant = 5
max_formant = 5500
window_length = 0.025
dyn_range = 50

# -----------------default display parameters
timer_begin = 0.0
timer_end = 0.0
max_freq = 5500

# ----------------read the selected sound
Show analyses... Yes No No Yes No 50

Formant settings... max_formant max_number_formant window_length dyn_range 1.0
f1v = Get first formant
f2v = Get second formant
f3v = Get third formant
# ----------------extract formant informations
Extract visible formant contour
startsound = Get begin of selection
endsound = Get end of selection
window = Get selection length

# ---------------- make the new sound name of the extract part
ss$ =  Get begin of selection
es$ = Get end of selection
ss$=left$(ss$,5)
es$=left$(es$,5)
ss$=replace$(ss$,".","s",0)
es$=replace$(es$,".","s",0)
soundname$ = originalsoundname$ + "--" + ss$ + "-" + es$

# ---------------- extract sound
Extract selected sound (preserve times)
# ---------------- end of the use of the editor
endeditor

# ---------------- rename sound and formant analysis
select Sound 'tempsoundname$'
Rename... 'soundname$'
select Formant 'tempsoundname$'
Rename... 'soundname$'

# ----------------read the total duration and sampling frequency of the sound
	select Sound 'soundname$'
	total_duration = Get total duration
	sampling_frequency = Get sampling frequency

# ----------------erase praat picture screen
	Erase all


# ----------------set the font and color
	Times
	Font size... 10
	Black
	Line width... 0.5

# ---------------- waveform
# ---------------- Define size and position of waveform (by specifying grid coordinates)
	select Sound 'soundname$'
	Viewport... 0 6 0 2
	Maroon
	Draw... 0 0 0 0 yes curve
	
# ----------------waveform labels creation
	Marks bottom every... 1 0.1 yes yes yes
 
#----------------formant analysis
select Formant 'soundname$'
Down to Table... no yes 2 no 3 no 2 no
tablename$ = selected$("Table")

# ----------------formant display
select Formant 'soundname$'
# ---------------- Speckle view
Viewport... 0 6 2 5
Speckle... timer_begin timer_end max_freq 30 yes
Marks bottom every... 1 0.1 yes yes yes
# ----------------  Tracks view
Viewport... 0 6 5 8
Draw tracks... timer_begin timer_end max_freq  yes
Marks bottom every... 1 0.1 yes yes yes
# ---------------- Scatter plot view
Viewport... 6 12 2 8

if scatter_plot$ = "F1 F2"
  scatter_vertical_formant = 1
  scatter_horizontal_formant = 2
  Scatter plot... timer_begin timer_end scatter_horizontal_formant 400 3000 scatter_vertical_formant 1500 100 1.0 yes +
endif

if scatter_plot$ = "F1 F3"
  scatter_vertical_formant = 1
  scatter_horizontal_formant = 3
  Scatter plot... timer_begin timer_end scatter_horizontal_formant 2000 4600 scatter_vertical_formant 1500 100 1.0 yes +
endif

if scatter_plot$ = "F2 F3"
  scatter_vertical_formant = 2
  scatter_horizontal_formant = 3
  Scatter plot... timer_begin timer_end scatter_horizontal_formant 2000 4600 scatter_vertical_formant 400 3000 1.0 yes +
endif

Marks left every... 1 100 yes yes yes
Marks bottom every... 1 200 yes yes yes

# ----------------write title
Font size... 18

Viewport... 6.5 7 0.35 0.5
Maroon
Text... 0 Left 0 top Formant Analysis of sound file:

Viewport... 6.5 7 0.5 1
Maroon
# Text top... no Sound file : 'soundname$' 
Text... 0 Left 0 top " 'soundname$' "

# ---------------- write informations
Font size... 12
Black
# ---------------- write title informations
Viewport... 6.5 7 1 1.25
Text... 0 Left 0 top Sampling frequency : 
Viewport... 6.5 7 1.25 1.5
Text... 0 Left 0 top Total duration: 
Viewport... 6.5 7 1.5 1.75
Text... 0 Left 0 top Start / End of selection: 
Viewport... 6.5 7 1.75 2
Text... 0 Left 0 top F1 / F2 / F3 (mean): 
# ---------------- write informations data
Viewport... 8.25 8.5 1 1.25
Text... 0 Left 0 top 'sampling_frequency' Hz
Viewport... 8.25 8.5 1.25 1.5
Text... 0 Left 0 top 'total_duration:3' s
Viewport... 8.25 8.5 1.5 1.75
Text... 0 Left 0 top 'startsound:3' s / 'endsound:3' s
Viewport... 8.25 8.5 1.75 2
Text... 0 Left 0 top 'f1v:0' Hz / 'f2v:0' Hz / 'f3v:0' Hz 

Font size... 10
Black

Maroon
# ---------------- write labels on each figure
	Select outer viewport... 5 5.5 0 0.4
	Text... 0 Right 0 Top  Waveform
	Select outer viewport... 5 5.5 2 2.4
	Text... 0 Right 0 Top  Speckle
	Select outer viewport... 5 5.5 5 5.4
	Text... 0 Right 0 Top  Tracks
	Select outer viewport... 11.5 11 2 2.4
	Text... 0 Right 0 Top  Scatter plot
Black

# ------------ save table (numerical data of the analysis
select Table 'tablename$'
if save_table_analysis$ = "Yes"
	select Table 'tablename$'
	Save as tab-separated file... 'analysis_output_directory$''soundname$'.xls
	echo Excel file (tab-separated format) "'soundname$'.xls" was written into " 'analysis_output_directory$' " directory
endif

# ------------ copy to clipboard or save on disk
	Viewport... 0 12 0 8
	if type_of_image$	= "Image (copy to clipboard)"
		Copy screen image to clipboard
		# echo Screen image is in the clipboard
	endif	
	if type_of_image$	= "Vectoriel (copy to clipboard)"
		Copy to clipboard
		# echo Vectored draw is in the clipboard
	endif	
	if type_of_image$	= "PostScript (write on disk)"
		Write to EPS file... 'analysis_output_directory$''soundname$'-fmtanalyse.eps
		echo File "'soundname$'-fmtanalyse.eps" was written into into " 'analysis_output_directory$' " directory
	endif	
	if type_of_image$	= "PDF (write on disk)"
		Write to PDF file... 'analysis_output_directory$''soundname$'-fmtanalyse.pdf
		echo File "'soundname$'-fmtanalyse.pdf" was written into into " 'analysis_output_directory$' " directory
	endif	
		
# ------------  clean object window
	select Formant 'soundname$'
	Remove
	select Table 'tablename$'
	Remove
	# select the original sound in the object window
	select Sound 'originalsoundname$'
